package xmlclx;
import java.util.*;
import java.sql.*;


public class DacJdbcDatabase 
{
    private boolean autoCommit;
    private Connection jdbcConnection;
    private String password;
    private int transIsolationLevel;
    private String urlConnection;
    private String user;
    private boolean useTableSchema;
    
    public DacJdbcDatabase() throws Exception
    {
        	
    }
    static String getSQLType(int sqlFieldType, int size)
    {
        switch (sqlFieldType)
        {
        	case Types.BIGINT:
        	    return "INTEGER"; // INT64
        	case Types.BIT:
        	case Types.BOOLEAN:
        	case Types.INTEGER:
        	case Types.SMALLINT:
        	case Types.TINYINT:
        	case Types.NUMERIC: // MONEY ?
        	    return "INTEGER";
        	
        	case Types.DECIMAL:
        	case Types.DOUBLE:
        	case Types.FLOAT:
        	case Types.REAL:
        	    return "FLOAT";
        	    
        	case Types.DATE:
        	case Types.TIME:
        	case Types.TIMESTAMP:
        	    return "DATE";

        	case Types.CHAR:
        	case Types.LONGVARCHAR:
        	case Types.VARCHAR:
        	    return "VARCHAR("+Integer.toString(size)+")";
        	
        	case Types.CLOB:
        	    return "MEMO";
            	
        	case Types.ARRAY:        	    
        	case Types.BINARY:
        	case Types.BLOB:
        	case Types.JAVA_OBJECT:
        	case Types.LONGVARBINARY:
        	case Types.STRUCT:
        	case Types.VARBINARY:
        	    return "BLOB"; 
        	
            default:
                return "VARCHAR("+Integer.toString(size)+")"; // by default
        }
    }
    public Connection getConnection()
    {
        return jdbcConnection;
    }
    public void close() throws Exception
	{
	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    jdbcConnection.setAutoCommit(true);
	    jdbcConnection.close();
	    jdbcConnection = null;
	}
	public void commit() throws Exception
	{
	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    if (!autoCommit)
	        jdbcConnection.commit();
	}
	public boolean getConnected()
	{
	    return jdbcConnection != null;
	}
	public void open() throws Exception
	{
	    if (user != null)
	        jdbcConnection = DriverManager.getConnection(urlConnection, user, password);
	    else
	        jdbcConnection = DriverManager.getConnection(urlConnection);
	    //DriverManager.setLogStream(java.lang.System.out);
	    //jdbcConnection.setAutoCommit(autoCommit);
	    jdbcConnection.setTransactionIsolation(transIsolationLevel);
	}
	public void rollback() throws Exception 
	{
	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    if (!autoCommit)
	        jdbcConnection.rollback();
	}
	public void setParams(String value) throws Exception
	{
        String autoCommitStr;
	    String driver;
	    int index;
	    String param;
	    String paramName;
	    String paramValue;
	    Map params = new HashMap();
	    String transIsolationStr;
	    String useTableSchemaStr;
	    String url;
	    
	    StringTokenizer st = new StringTokenizer(value, ",");
	    while (st.hasMoreTokens())
	    {
	        param = st.nextToken();
	        index = param.indexOf("=");
	        paramName = param.substring(0, index);
            paramValue = param.substring(index+1);
	        params.put(paramName, paramValue);
	    }
	    driver = (String)params.get("Driver");
	    Class.forName(driver);
	    
	    urlConnection = (String) params.get("ConnectionURL");
        autoCommitStr = (String)params.get("AutoCommit");
        useTableSchemaStr = (String)params.get("UseTableSchema");
	    autoCommit = (autoCommitStr != null) && (autoCommitStr.compareToIgnoreCase("true") == 0);
	    user = (String) params.get("User");
	    password = (String) params.get("Password");
	    useTableSchema = (useTableSchemaStr != null) && (useTableSchemaStr.compareToIgnoreCase("true") == 0);
	    transIsolationStr = (String)params.get("TransactionIsolation");
	    if (transIsolationStr.compareToIgnoreCase("TRANSACTION_READ_COMMITTED") == 0)
	        transIsolationLevel = Connection.TRANSACTION_READ_COMMITTED;
	    else if (transIsolationStr.compareToIgnoreCase("TRANSACTION_READ_UNCOMMITTED") == 0)
	        transIsolationLevel = Connection.TRANSACTION_READ_UNCOMMITTED;
	    else if (transIsolationStr.compareToIgnoreCase("TRANSACTION_REPEATABLE_READ") == 0)
	        transIsolationLevel = Connection.TRANSACTION_REPEATABLE_READ;
	    else if (transIsolationStr.compareToIgnoreCase("TRANSACTION_SERIALIZABLE") == 0)
	        transIsolationLevel = Connection.TRANSACTION_SERIALIZABLE;
	    else 
	        transIsolationLevel = Connection.TRANSACTION_NONE; // default value
	}
	public void startTransaction() throws Exception
	{
	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    if (!autoCommit)
	        jdbcConnection.setAutoCommit(autoCommit);
	}
	public String getTableNames() throws Exception
	{
	    DatabaseMetaData metaData;
	    String tableName;
	    ResultSet tableNames;
	    ResultSet tableTypes;
	    String[] names = {"TABLE"};
	    StringBuffer res;
	    
	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    jdbcConnection.setAutoCommit(true);//
	    metaData = jdbcConnection.getMetaData();
	    //tableNames = metaData.getTables(null, "%", "%", names);
	    tableNames = metaData.getTables(null, null, null, names);
	    res = new StringBuffer();
	    while (tableNames.next()) 
	    {
	        if (res.length() != 0)
	            res.append(",");
	        res.append("\"");
	        tableName = tableNames.getString("TABLE_NAME");
	        if (useTableSchema)
	            tableName = tableNames.getString("TABLE_SCHEM") + "." + tableName;
	        res.append(tableName);
	        res.append("\"");
	    }
	    return res.toString();
	}
	public String getIndexDefs(String table) throws Exception
	{
	    ResultSet indices;
	    DatabaseMetaData metaData;
	    StringBuffer res;
	    String schema;

	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    metaData = jdbcConnection.getMetaData();
	    schema = "%";
	    if (useTableSchema)
	    {
            int index = table.indexOf(".");
            schema = table.substring(0, index);
            table = table.substring(index + 1);
	    }
	    indices = metaData.getIndexInfo(null, schema, table, false, false);   
	    res = new StringBuffer();
	    while (indices.next()) 
	    {
	        if (res.length() != 0)
	            res.append(",");
	        res.append(indices.getString("INDEX_NAME"));    
	    }
	    return res.toString();
	}
	public String getFieldDefs(String table) throws Exception 
	{
	    ResultSet columns;
	    DatabaseMetaData metaData;
	    StringBuffer res;
	    int sqlFieldType;
	    String fieldType;
	    String schema;

	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    schema = null;
        if (useTableSchema) 
        {
            int index = table.indexOf(".");
            schema = table.substring(0, index);
            table = table.substring(index + 1);
        }	    
	    metaData = jdbcConnection.getMetaData();
	    if (urlConnection.toLowerCase().indexOf("interbase") > 0)
	    	columns = metaData.getColumns(null, "%", table, "%"); // Interbase
	    else
	    	columns = metaData.getColumns(null, schema, table, null); // Access
	    res = new StringBuffer();
	    while (columns.next())
	    {
	        if (res.length() != 0)
	            res.append(",");
	        res.append("\"");
	        res.append(columns.getString("COLUMN_NAME"));
	        res.append("=");
	        sqlFieldType = columns.getInt("DATA_TYPE");
	        fieldType = getSQLType(sqlFieldType, columns.getInt("COLUMN_SIZE"));
	        res.append(fieldType); 
	        res.append("\"");
	    }
	    return res.toString();
	}
	public String getProcedureNames(String packageName) throws Exception 
	{
	    ResultSet procedures;
	    DatabaseMetaData metaData;
	    StringBuffer res;

	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    metaData = jdbcConnection.getMetaData();
	    procedures = metaData.getProcedures(null, "%", "%");	    
	    res = new StringBuffer();
	    while (procedures.next())
	    {
	        if (res.length() != 0)
	            res.append(",");
	        res.append(procedures.getString("PROCEDURE_NAME"));
	    }
	    return res.toString();	    
	}
	public void forceCommit() throws Exception
	{
	    commit();
	}

}
