package xmlclx;
import java.sql.*;
import java.util.*;
import xmlclx.DacJdbcDatabase;


public class DacJdbcQuery
{
    private String commandType;
    private boolean eof;
    private Connection jdbcConnection;
    private Map params;
    private ResultSet resultSet;
    private String sql;
    private PreparedStatement statement;
    
    private boolean isLetterOrDigit(char c)
    {
        if ((c >= 'A') && (c <= 'Z')) 
            return true;
        if ((c >= 'a') && (c <= 'z'))
            return true;
        if ((c >= '0') && (c <= '9'))
            return true;
        if (c == '_')
            return true;
    	return false;
    }
    private String prepareSql(String sql)
    {
        int index, indexStart, indexEnd, indexParam;
        String paramName;
        StringBuffer preparedSql;
        
        preparedSql = new StringBuffer();
        indexStart = 0;
        indexParam = 1; // begins at 1 !
        index = sql.indexOf(":", indexStart);
        while (index > -1)
        {
            preparedSql.append(sql.substring(indexStart, index));
            preparedSql.append("?");
            index++;
            indexEnd = index;            
            while ((indexEnd < sql.length()) && (isLetterOrDigit(sql.charAt(indexEnd))))
                indexEnd++;
            paramName = sql.substring(index, indexEnd);
            params.put(paramName, new Integer(indexParam));
            indexParam++;
            indexStart = indexEnd;
            index = sql.indexOf(":", indexStart);
        }
        preparedSql.append(sql.substring(indexStart));
        return preparedSql.toString();
    }
    
    public DacJdbcQuery()
    {
        params = new LinkedHashMap();
        commandType = "TEXT";
    }
	public void close() throws Exception
	{
	    if (resultSet != null)
	    {
	        resultSet.close();
	        resultSet = null;
	    }
	}
	public void execSQL() throws Exception
	{
	    if (statement == null)
	        prepare();
	    statement.executeUpdate();   
	}
	public boolean eof()
	{
	    return eof;
	}
	public int fieldCount() throws Exception
	{
	    if (resultSet == null)
	        return 0;
	    if (eof)
	        return 0;
	    return resultSet.getMetaData().getColumnCount();
	}
	public boolean getActive()
	{
	    return resultSet != null;
	}
	public String getFieldAsString(String fieldName) throws Exception
	{
	    if (resultSet == null)
	        throw new Exception("resultSet is null");
	    if (!eof)
	    	return resultSet.getString(fieldName);
	   else 
	   		return "";
	}
	public String getFieldName(int index) throws Exception
	{
	    if (resultSet == null)
	        throw new Exception("resultSet is null");
	    return resultSet.getMetaData().getColumnName(index+1); // Begins at 1	    
	}
	public String getFieldNoAsString(int index) throws Exception
	{
	    if (resultSet == null)
	        throw new Exception("resultSet is null");
	    if (!eof)
	    	return resultSet.getString(index+1); // Begins at 1
	    else
	    	return "";
	}
	public String getParamName(int index) throws Exception 
	{
	    
	    if (statement == null)
	        throw new Exception("statement is null");
	    Iterator it = params.keySet().iterator();
	    String paramName = "";
	    for (int i = 0; i <= index; i++)
	        paramName = (String) it.next();
	    return paramName;
	}
	public String getParamAsString(String paramName) throws Exception
	{
	    throw new Exception("getParamAsString not supported");
	}
	public String getParamNoAsString(int index) throws Exception
	{
	    throw new Exception("getParamNoAsString not supported");
	}
	public void next() throws Exception
	{
	    if (resultSet == null)
	        throw new Exception("resultSet is null");
	    eof = !resultSet.next();
	}
	public void open() throws Exception
	{
	    if (statement == null)
	        prepare();
	    resultSet = statement.executeQuery();
	    eof = !resultSet.next();
	}
	public int paramCount() throws Exception
	{
	    if (statement == null)
	        throw new Exception("statement is null");
	    return params.size();
	    //return statement.getParameterMetaData().getParameterCount(); 
	}
	public void prepare() throws Exception
	{
	    String preparedSql;
	    
	    if (jdbcConnection == null)
	        throw new Exception("jdbcConnection is null");
	    preparedSql = prepareSql(sql);
	    statement = jdbcConnection.prepareStatement(preparedSql);
	    
	}
	public boolean prepared()
	{
	    return statement != null;
	}
	public void setDatabase(DacJdbcDatabase database)
	{
	    jdbcConnection = database.getConnection();
	    statement = null;
	}
	public void setFieldAsString(String fieldName, String value) throws Exception
	{
	    throw new Exception("setFieldAsString not supported");
	}
	public void setFieldNoAsString(int index, String value) throws Exception
	{
	    throw new Exception("setFieldNoAsString not supported");
	}
	public void setParamAsString(String paramName, String value) throws Exception
	{
	    int paramIndex;
	    
	    if (statement == null)
	        prepare();
	    paramIndex = ((Integer)params.get(paramName)).intValue();
	    statement.setString(paramIndex, value);    
	}
	public void setParamNoAsString(int index, String value) throws Exception
	{
	    if (statement == null)
	        prepare();
	    statement.setString(index+1, value); // Begins at 1     
	}
	public void setSQL(String value)
	{
	    sql = value; 
	    params.clear();
	    statement = null;
	}
	public void setParamTypeNo(int index, String value) throws Exception
	{
	    // prepare must type paramters
	}
	public void setParamType(String paramName, String value)
	{
	    // prepare must type paramters
	}
	public String getFieldNoType(int index) throws Exception
	{
	    int sqlFieldType;
	    
	    if (resultSet == null)
	        throw new Exception("resultSet is null");
	    sqlFieldType = resultSet.getMetaData().getColumnType(index+1); // Begins at 1 
	    return DacJdbcDatabase.getSQLType(sqlFieldType, resultSet.getMetaData().getColumnDisplaySize(index+1));	    
	}
	public String getFieldType(String fieldName) throws Exception
	{
	    int fieldIndex;
	    int sqlFieldType;
	    
	    if (resultSet == null)
	        throw new Exception("resultSet is null");
	    fieldIndex = resultSet.findColumn(fieldName);
	    sqlFieldType = resultSet.getMetaData().getColumnType(fieldIndex); 
	    return DacJdbcDatabase.getSQLType(sqlFieldType, resultSet.getMetaData().getColumnDisplaySize(fieldIndex));	    
	}
	public String getParamNoType(int index) throws Exception
	{
	    int sqlFieldType;
	    
	    if (resultSet == null)
	        throw new Exception("resultSet is null");
	    sqlFieldType = statement.getParameterMetaData().getParameterType(index+1); // Begins at 1
	    return DacJdbcDatabase.getSQLType(sqlFieldType, 0); 	    
	}
	public String getParamType(String paramName) throws Exception
	{
	    int paramIndex; 
	    
	    paramIndex = ((Integer)params.get(paramName)).intValue();
	    return statement.getParameterMetaData().getParameterTypeName(paramIndex);
	}
	public String getCommandType()
	{
	    return commandType;
	}
	public void setCommandType(String value)
	{
	    commandType = value;
	}
}
