package xmlclx;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.apache.xpath.XPathAPI;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

public class JavaXMLCursor
{
    Document xmlDocument;     
    Node currentNode;          
    NodeList currentNodeList;  
    boolean eof; 
    int position;  

    
    public JavaXMLCursor()
    {
        xmlDocument = null;     
        currentNode = null;          
        currentNodeList = null;  
        eof = true; 
        position = -1;  
    }
    
    private Document createXmlDocument() throws Exception 
    {
        Document document = null;
        
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
        document = documentBuilder.newDocument();
        return document;
    }
    private Document createXmlDocumentFromString(String sourceXML) throws Exception
    {
        Document document = null;
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
        document = documentBuilder.parse(new InputSource(new StringReader(sourceXML)));
        return document;
    }
    private Document createXmlDocumentFromFile(String fileName) throws Exception
    {
        Document document = null;
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
        document = documentBuilder.parse(new InputSource(new FileReader(fileName)));
        return document;
    }
    NodeList getCurrentNodeList()
    {
        return currentNodeList;
    }
    private NodeList internalAppendChild(String elementName, String value, NodeList nodeList) throws Exception 
    {
        Element element = null;

        if (xmlDocument == null) 
        {
          xmlDocument = createXmlDocument();
          element = xmlDocument.createElement(elementName);
          xmlDocument.appendChild(element);
          if ((value != null) && (!value.equals("")))
            element.appendChild(xmlDocument.createTextNode(value));
          currentNodeList = xmlDocument.getChildNodes();
          nodeList = currentNodeList;
          first();
        } 
        else 
        {
          if (currentNode == null)
            throw new Exception("XMLCursor.internalAppendChild - EOF");
          element = xmlDocument.createElement(elementName);
          currentNode.appendChild(element);
          if ((value != null) && (!value.equals("")))
            element.appendChild(xmlDocument.createTextNode(value));
          nodeList = currentNode.getChildNodes();
        }
        return nodeList;
    }

    public JavaXMLCursor appendChild(String elementName, String value) throws Exception
	{
        JavaXMLCursor xmlCursor = null; // the new XMLCursor
        NodeList nodeList = null;   // the new XMLCursor's nodelist

        nodeList = internalAppendChild(elementName, value, nodeList);
        xmlCursor = new JavaXMLCursor();
        xmlCursor.xmlDocument = xmlDocument;
        xmlCursor.currentNodeList = nodeList;
        xmlCursor.last(); 
        return xmlCursor;
	}
	public void appendChild2(String elementName, String value) throws Exception
	{
	    internalAppendChild(elementName, value, null);
	}
	public void appendXMLCursor(JavaXMLCursor xmlCursor) throws Exception
	{
	    NodeList nodeList = null;
	    Node container = null;
	    Node node = null;

	    if (xmlCursor == null)
	        return;  
	    nodeList = xmlCursor.currentNodeList;
	    if (nodeList == null) 
	        return;
	    if (nodeList.getLength() == 0)
	        return;
	    if (xmlDocument == null) 
	    {
	        if (nodeList.getLength() > 1)
	            throw new Exception("XMLCursor.appendXMLCursor - Cannot add XMLCursor to current since it has more than one element");
	        xmlDocument = createXmlDocument();
	        xmlDocument.appendChild(xmlDocument.importNode(nodeList.item(0), true));
	        currentNodeList = xmlDocument.getChildNodes();
	        nodeList = currentNodeList;
	        first();
	        return;
	    }
	    if (currentNode == null)
	        return;
	    container = currentNode;
	    for (int i = 0; i < nodeList.getLength(); i++)
	    {
	        node = xmlDocument.importNode(nodeList.item(i), true);
	        container.appendChild(node);
	    }
	}
	public String containerXML() throws Exception
	{
	    if (currentNode == null)
	        return "";
	    if (currentNode.getParentNode() == null)
	    	return xml();
	    else
	    {
	        Transformer serializer = null;
	        StringWriter result = new StringWriter();
	        
	        serializer = TransformerFactory.newInstance().newTransformer();
	        serializer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
	        serializer.setOutputProperty(OutputKeys.INDENT, "yes");
	        serializer.transform(new DOMSource(currentNode.getParentNode()), new StreamResult(result));
	        return result.toString();
	    }
	}
	public int count()
	{
	    if (currentNodeList == null)
	        return 0;
	    return currentNodeList.getLength();
	}
	public void delete()
	{
	    Node parentNode;
	    
	    if (currentNode == null)
	        return;
	    parentNode = currentNode.getParentNode();
	    if (parentNode == null)
	        return;
	    parentNode.removeChild(currentNode);
	    if (currentNodeList == null)
	    {
	        position = -1;
	        currentNode = null;
	        eof = true;
	        return;
	    }
	    if (position >= currentNodeList.getLength())
	        position = currentNodeList.getLength()-1;
	    currentNode = currentNodeList.item(position);
	}
	public JavaXMLCursor document() throws Exception
	{
	    JavaXMLCursor newCursor;
	  
	    if (xmlDocument == null)
	      return null;
	    newCursor = new JavaXMLCursor();
	    newCursor.xmlDocument = xmlDocument;
	    newCursor.currentNodeList = xmlDocument.getChildNodes();
	    newCursor.first();
	    return newCursor;	    
	}
	public boolean eof()
	{
	    return (eof || (position == -1));
	}
	public void first() throws Exception
	{
	    moveTo(0);
	}
	public String getName()
	{
	    if (currentNode == null) 
	        return "";
	    return currentNode.getNodeName();
	}
	public String getValue(String xpath) throws Exception
	{
	    Node node = null;
	    if (currentNode == null)
	        return "";
	    node = XPathAPI.selectSingleNode(currentNode, xpath);
	    if (node == null) 
	        return "";
	    if (node.getNodeType() == Node.ATTRIBUTE_NODE)
	        return node.getNodeValue();
	    else
	        node = XPathAPI.selectSingleNode(node, "text()");
	    if (node == null)  
	        return "";
	    return node.getNodeValue();
	}
	public JavaXMLCursor insertBefore(String elementName, String value) throws Exception
	{
	    JavaXMLCursor newCursor;
	    Element element;
	    Node node;
	    Node parentNode;
	    int pos;

	    if (xmlDocument == null)
	      return appendChild(elementName, value);
	    if (currentNode == null)
	      return null;
	    parentNode = currentNode.getParentNode();
	    if (parentNode == null)
	      return null;
	    element = xmlDocument.createElement(elementName);
	    parentNode.insertBefore(element, currentNode);
	    if (!value.equals(""))
	    {
	        node = xmlDocument.createTextNode(value);
	        element.appendChild(node);
	    }
	    newCursor = new JavaXMLCursor();
	    newCursor.xmlDocument = xmlDocument;
	    newCursor.currentNodeList = parentNode.getChildNodes();
	    node = currentNode;
	    // retrieve the absolute position of FCurrentNode in parentNode.childNodes
	    pos = 0;
	    while (true)
	    {
	        node = node.getPreviousSibling();
	        if (node == null)
	            break;
	        pos++;
	    }
	    if (pos > 0) // except when creation of the document
	      pos--;
	    newCursor.moveTo(pos);
	    return newCursor;
	}
	public JavaXMLCursor insertAfter(String elementName, String value) throws Exception
	{
	    JavaXMLCursor newCursor;
	    Element element;
	    Node node;
	    Node parentNode;
	    int pos;
	  
	    if (xmlDocument == null)
	      return appendChild(elementName, value);
	    if (currentNode == null)
	      return null;
	    parentNode = currentNode.getParentNode();
	    if (parentNode == null)
	      return null;
	    element = xmlDocument.createElement(elementName);
	    node = currentNode.getNextSibling();
	    if (node == null)
	        parentNode.appendChild(element);
	    else
	        parentNode.insertBefore(element, node);
	    if (!value.equals(""))
	    {
	        node = xmlDocument.createTextNode(value);
	        element.appendChild(node);
	    }
	    newCursor = new JavaXMLCursor();
	    newCursor.xmlDocument = xmlDocument;
	    newCursor.currentNodeList = parentNode.getChildNodes();
	    node = currentNode;
	    // retrieve the absolute position of FCurrentNode in parentNode.childNodes
	    pos = 0;
	    while (true)
	    {
	        node = node.getPreviousSibling();
	        pos++;
	        if (node == null)
	            break;
	    }
	    newCursor.moveTo(pos);
	    return newCursor;
	}
	public JavaXMLCursor insertFirstChild(String elementName, String value) throws Exception
	{
	    Element element;
	    JavaXMLCursor newCursor;
	    Node node;

	    if (xmlDocument == null)
	      return appendChild(elementName, value);
	    if (currentNode == null)
	      return null;
	    node = currentNode.getFirstChild();
	    element = xmlDocument.createElement(elementName);
	    currentNode.insertBefore(element, node);
	    if (!value.equals("")) 
	    {
	        node = xmlDocument.createTextNode(value);
	        element.appendChild(node);
	    }
	    newCursor = new JavaXMLCursor();
	    newCursor.xmlDocument = xmlDocument;
	    newCursor.currentNodeList = currentNode.getChildNodes();
	    newCursor.first();
	    return newCursor;
	}
	public void last() throws Exception
	{
	    moveTo(count() - 1);
	}
	public void load(String fileName) throws Exception
	{
        xmlDocument = createXmlDocumentFromFile(fileName);
	    if (xmlDocument.getDocumentElement() == null)
	        currentNodeList = null;
	    else
	        currentNodeList = xmlDocument.getChildNodes();
	    first(); // To be sure to be on the document element
	}
	public void loadXML(String sourceXML) throws Exception
	{
	    xmlDocument = createXmlDocumentFromString(sourceXML);
	    if (xmlDocument.getDocumentElement() == null )  
	      currentNode = null;
	    else
	      currentNodeList = xmlDocument.getChildNodes();
	    first();
	}
	public void moveTo(int index) throws Exception
	{
	    int vCount;

	    if (currentNodeList == null)
	      return;
	    vCount = count(); 
	    if (vCount == 0) 
	      return;
	    if ((index < 0) || (index >= vCount))
	      throw new Exception("XMLCursor.moveTo - Index out of range " + index + "/" + vCount);
	    if (currentNodeList == null) {
	      return;
	    }
	    position = index;
	    eof = false;
	    currentNode = currentNodeList.item(position);
	}
	public void next()
	{
	    if ((count() - 1 == position) || (currentNode == null)) 
	    {
	        eof = true;
	        return;
	    }
	    position++;
	    currentNode = currentNodeList.item(position);
	}
	public String prettyXML() throws Exception
	{
	    return xml();
	}
	public int recNo()
	{
	    return position;
	}
	public void replaceWithXMLCursor(JavaXMLCursor xmlCursor) throws Exception
	{
	    Node clonedNode;
	  	int count;
	  	NodeList nodeList;
	  	Node parentNode;
	  	
	    if (xmlDocument == null)
	        throw new Exception("JavaXMLCursor.ReplaceWithXMLCursor - FXMLDOMDocument = nil");
	    if (currentNode == null)
	        return;
	    parentNode = currentNode.getParentNode();
	    if (parentNode == null)
	        return;
	    if (xmlCursor == null)
	    	return;
	    nodeList = xmlCursor.getCurrentNodeList();
	  count = nodeList.getLength();
	  for (int i = 0; i < count; i++)
	  {
	    clonedNode = xmlDocument.importNode(nodeList.item(i), true);
	    parentNode.insertBefore(clonedNode, currentNode);
	  }
	  currentNodeList = parentNode.getChildNodes();
	  parentNode.removeChild(currentNode);
	  currentNode = null;
	  first(); // Reset FPosition
	}
	public void save(String fileName) throws Exception
	{
        Transformer serializer = null;
        FileWriter result = new FileWriter(fileName);
        
        serializer = TransformerFactory.newInstance().newTransformer();
        serializer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        serializer.setOutputProperty(OutputKeys.INDENT, "yes");
        serializer.transform(new DOMSource(xmlDocument), new StreamResult(result));
	}
	public JavaXMLCursor select(String xpath) throws Exception
	{
	    NodeList nodeList = null;               
	    JavaXMLCursor xmlCursor = new JavaXMLCursor();  

	    if (currentNode == null) 
	    {
	        xmlCursor.first();
	        return xmlCursor;
	    }
	    //if (xpath.equals("*"))
	        //nodeList = currentNode.getChildNodes();
	      //else
	    nodeList = XPathAPI.selectNodeList(currentNode, xpath);
	    xmlCursor.xmlDocument = xmlDocument;
	    xmlCursor.currentNodeList = nodeList;
	    xmlCursor.first();
	    return xmlCursor;
	}
	public void setAttributeValue(String attributeName, String value)
	{
	    Element element;

	    if (currentNode == null)
	      return;
	    element = (Element) currentNode;
	    element.setAttribute(attributeName, value);
	}
	public void setCValue(String elementName, String value) throws Exception
	{
	    Node node;
	    Node cdata;

	    if (currentNode == null)
	      return;
	    node = XPathAPI.selectSingleNode(currentNode, elementName);
	    if (node == null)
	    {
	        node = xmlDocument.createElement(elementName);
	        currentNode.appendChild(node);
	    }
	    cdata = node.getFirstChild();
	    if (cdata == null)
	    {
	    	// Should protect CDATA against ]]>
	        cdata = xmlDocument.createCDATASection(value);
	        node.appendChild(cdata);
	        return; 
	    } 
	    else 
	    {
	        // Should protect CDATA against ]]>
	        cdata.setNodeValue(value);
	    }
	}
	public void setValue(String elementName, String value) throws Exception
	{
	    String attributeName;
	  	Node node;
	  	Node textNode;
	  	
	  	
	  	if (currentNode == null)
	  	    return;
	  	if (elementName.startsWith("@"))
	  	{
	  	    attributeName = elementName.substring(1);
			setAttributeValue(attributeName, value);
			return;
	  	}
	  	node = XPathAPI.selectSingleNode(currentNode, elementName);
	  	if (node == null)
	  	{	
	  	    node = xmlDocument.createElement(elementName);
	  	    currentNode.appendChild(node);
	  	}
	  	else if (node.getNodeType() == Node.TEXT_NODE)
	  	{
	  	  node.setNodeValue(value);
	  	  return;
	  	}
	  	textNode = node.getFirstChild();
	  	if ((textNode != null) && (textNode.getNodeType() == Node.TEXT_NODE))
	  	  textNode.setNodeValue(value);
	  	else
	  	{
	  	    //node = textNode;
	  	    textNode = xmlDocument.createTextNode(value);
	  	    node.appendChild(textNode);
	  	}
	}
	public String xml() throws Exception
	{
        Transformer serializer = null;
        StringWriter result = new StringWriter();
        
        serializer = TransformerFactory.newInstance().newTransformer();
        serializer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
        serializer.setOutputProperty(OutputKeys.INDENT, "yes");
        serializer.transform(new DOMSource(currentNode), new StreamResult(result));
        return result.toString();
	}
}
