package xmlclx;

import java.lang.reflect.*;

public class XMLCollectionEvents
{
	public static class AfterDispatchEventArgs
	{
	}
	public static class BeforeDispatchEventArgs
	{
		public boolean handled;
	}
	public static class AuthenticateEventArgs
	{
		public boolean handled;
	}
	public static class AuthorizeEventArgs
	{
		public boolean handled;
		public String xmlService;
	}
	public static class ByPassEventArgs
	{
		public boolean byPass;
	}
	public static class ExceptionEventArgs
	{
		public boolean handled;
		public String message;
	}
	public static class ExecuteXMLServiceEventArgs
	{
		public boolean skip;
		public String xmlServiceName;
		public XMLService xmlService;
	}
	public static class InitializeEventArgs
	{
	}
	
	/*
	 * constant types 
	 */
	
	final private Class[] afterDispatchTypes = {XMLCollection.class, AfterDispatchEventArgs.class};
	final private Class[] beforeDispatchTypes = {XMLCollection.class, BeforeDispatchEventArgs.class};
	final private Class[] authenticateTypes = {XMLCollection.class, AuthenticateEventArgs.class};
	final private Class[] authorizeTypes = {XMLCollection.class, AuthorizeEventArgs.class};
	final private Class[] byPassTypes = {XMLCollection.class, ByPassEventArgs.class};
	final private Class[] exceptionTypes = {XMLCollection.class, ExceptionEventArgs.class};
	final private Class[] executeXMLServiceTypes = {XMLCollection.class, ExecuteXMLServiceEventArgs.class};
	final private Class[] initializeTypes = {XMLCollection.class, InitializeEventArgs.class};
	
	private Method afterDispatchEventHandler;
	private Method beforeDispatchEventHandler;
	private Method onAuthenticateEventHandler;
	private Method onAuthorizeEventHandler;
	private Method onByPassEventHandler;
	private Method onExceptionEventHandler;
	private Method onExecuteXMLServiceEventHandler;
	private Method onInitializeEventHandler;

	public XMLCollectionEvents()
	{
	}
	public void addAfterDispatch(XMLModule xmlModule, String methodName) throws NoSuchMethodException
	{
		afterDispatchEventHandler = xmlModule.getClass().getMethod(methodName, afterDispatchTypes);
	}
	public void addBeforeDispatch(XMLModule xmlModule, String methodName) throws NoSuchMethodException
	{
		beforeDispatchEventHandler = xmlModule.getClass().getMethod(methodName, beforeDispatchTypes);
	}
	public void addOnAuthenticate(XMLModule xmlModule, String methodName) throws NoSuchMethodException
	{
		onAuthenticateEventHandler = xmlModule.getClass().getMethod(methodName, authenticateTypes);
	}
	public void addOnAuthorize(XMLModule xmlModule, String methodName) throws NoSuchMethodException
	{
		onAuthorizeEventHandler = xmlModule.getClass().getMethod(methodName, authorizeTypes);
	}
	public void addOnByPass(XMLModule xmlModule, String methodName) throws NoSuchMethodException
	{
		onByPassEventHandler = xmlModule.getClass().getMethod(methodName, byPassTypes); 
	}
	public void addOnException(XMLModule xmlModule, String methodName) throws NoSuchMethodException
	{
		onExceptionEventHandler = xmlModule.getClass().getMethod(methodName, exceptionTypes);
	}
	public void addOnExecuteXMLService(XMLModule xmlModule, String methodName) throws NoSuchMethodException
	{
		onExecuteXMLServiceEventHandler = xmlModule.getClass().getMethod(methodName, executeXMLServiceTypes);
	}
	public void addOnInitialize(XMLModule xmlModule, String methodName) throws NoSuchMethodException
	{
		onInitializeEventHandler = xmlModule.getClass().getMethod(methodName, initializeTypes);
	}
}
